@@ -13,7 +13,7 @@ import tempfile
 try:  # pragma: no cover
     from importlib import metadata
 except ImportError:  # pragma: no cover
-    import importlib_metadata as metadata
+    import importlib_metadata as metadata  # type: ignore
 
 from pyrsistent import m
 
@@ -29,7 +29,7 @@ from jsonschema.validators import _LATEST_VERSION, validate
 def fake_validator(*errors):
     errors = list(reversed(errors))
 
-    class FakeValidator(object):
+    class FakeValidator:
         def __init__(self, *args, **kwargs):
             pass
 
@@ -697,6 +697,7 @@ class TestCLI(TestCase):
 
     def test_successful_validation_via_explicit_base_uri(self):
         ref_schema_file = tempfile.NamedTemporaryFile(delete=False)
+        ref_schema_file.close()
         self.addCleanup(os.remove, ref_schema_file.name)
 
         ref_path = Path(ref_schema_file.name)
@@ -717,6 +718,7 @@ class TestCLI(TestCase):
 
     def test_unsuccessful_validation_via_explicit_base_uri(self):
         ref_schema_file = tempfile.NamedTemporaryFile(delete=False)
+        ref_schema_file.close()
         self.addCleanup(os.remove, ref_schema_file.name)
 
         ref_path = Path(ref_schema_file.name)
