@@ -1,19 +1,27 @@
+from __future__ import annotations
+
 from collections import deque, namedtuple
 from contextlib import contextmanager
 from decimal import Decimal
 from io import BytesIO
-from unittest import TestCase
+from unittest import TestCase, mock
 from urllib.request import pathname2url
 import json
 import os
 import sys
 import tempfile
 import unittest
+import warnings
 
-from twisted.trial.unittest import SynchronousTestCase
 import attr
 
-from jsonschema import FormatChecker, TypeChecker, exceptions, validators
+from jsonschema import (
+    FormatChecker,
+    TypeChecker,
+    exceptions,
+    protocols,
+    validators,
+)
 from jsonschema.tests._helpers import bug
 
 
@@ -23,7 +31,7 @@ def fail(validator, errors, instance, schema):
         yield exceptions.ValidationError(**each)
 
 
-class TestCreateAndExtend(SynchronousTestCase):
+class TestCreateAndExtend(TestCase):
     def setUp(self):
         self.addCleanup(
             self.assertEqual,
@@ -189,6 +197,38 @@ class TestCreateAndExtend(SynchronousTestCase):
             ),
         )
 
+    def test_check_schema_with_different_metaschema(self):
+        """
+        One can create a validator class whose metaschema uses a different
+        dialect than itself.
+        """
+
+        NoEmptySchemasValidator = validators.create(
+            meta_schema={
+                "$schema": validators.Draft202012Validator.META_SCHEMA["$id"],
+                "not": {"const": {}},
+            },
+        )
+        NoEmptySchemasValidator.check_schema({"foo": "bar"})
+
+        with self.assertRaises(exceptions.SchemaError):
+            NoEmptySchemasValidator.check_schema({})
+
+        NoEmptySchemasValidator({"foo": "bar"}).validate("foo")
+
+    def test_check_schema_with_different_metaschema_defaults_to_self(self):
+        """
+        A validator whose metaschema doesn't declare $schema defaults to its
+        own validation behavior, not the latest "normal" specification.
+        """
+
+        NoEmptySchemasValidator = validators.create(
+            meta_schema={"fail": [{"message": "Meta schema whoops!"}]},
+            validators={"fail": fail},
+        )
+        with self.assertRaises(exceptions.SchemaError):
+            NoEmptySchemasValidator.check_schema({})
+
     def test_extend(self):
         original = dict(self.Validator.VALIDATORS)
         new = object()
@@ -261,18 +301,6 @@ class TestValidationErrorMessages(TestCase):
         message = self.message_for(instance=1, schema={"type": list(types)})
         self.assertEqual(message, "1 is not of type 'string', 'object'")
 
-    def test_object_without_title_type_failure(self):
-        type = {"type": [{"minimum": 3}]}
-        message = self.message_for(
-            instance=1,
-            schema={"type": [type]},
-            cls=validators.Draft3Validator,
-        )
-        self.assertEqual(
-            message,
-            "1 is not of type {'type': [{'minimum': 3}]}",
-        )
-
     def test_object_with_named_type_failure(self):
         schema = {"type": [{"name": "Foo", "minimum": 3}]}
         message = self.message_for(
@@ -300,6 +328,18 @@ class TestValidationErrorMessages(TestCase):
         )
         self.assertEqual(message, "'foo' is a dependency of 'bar'")
 
+    def test_object_without_title_type_failure_draft3(self):
+        type = {"type": [{"minimum": 3}]}
+        message = self.message_for(
+            instance=1,
+            schema={"type": [type]},
+            cls=validators.Draft3Validator,
+        )
+        self.assertEqual(
+            message,
+            "1 is not of type {'type': [{'minimum': 3}]}",
+        )
+
     def test_dependencies_list_draft3(self):
         depend, on = "bar", "foo"
         schema = {"dependencies": {depend: [on]}}
@@ -586,9 +626,14 @@ class TestValidationErrorMessages(TestCase):
         message = self.message_for(instance=["foo", "bar"], schema=schema)
         self.assertIn(
             message,
-            "Unevaluated items are not allowed ('foo', 'bar' were unexpected)",
+            "Unevaluated items are not allowed ('bar', 'foo' were unexpected)",
         )
 
+    def test_unevaluated_items_on_invalid_type(self):
+        schema = {"type": "array", "unevaluatedItems": False}
+        message = self.message_for(instance="foo", schema=schema)
+        self.assertEqual(message, "'foo' is not of type 'array'")
+
     def test_unevaluated_properties(self):
         schema = {"type": "object", "unevaluatedProperties": False}
         message = self.message_for(
@@ -601,12 +646,17 @@ class TestValidationErrorMessages(TestCase):
         self.assertEqual(
             message,
             "Unevaluated properties are not allowed "
-            "('foo', 'bar' were unexpected)",
+            "('bar', 'foo' were unexpected)",
         )
 
+    def test_unevaluated_properties_on_invalid_type(self):
+        schema = {"type": "object", "unevaluatedProperties": False}
+        message = self.message_for(instance="foo", schema=schema)
+        self.assertEqual(message, "'foo' is not of type 'object'")
+
 
 class TestValidationErrorDetails(TestCase):
-    # TODO: These really need unit tests for each individual validator, rather
+    # TODO: These really need unit tests for each individual keyword, rather
     #       than just these higher level tests.
     def test_anyOf(self):
         instance = 5
@@ -1388,11 +1438,11 @@ class TestValidationErrorDetails(TestCase):
         )
 
 
-class MetaSchemaTestsMixin(object):
+class MetaSchemaTestsMixin:
     # TODO: These all belong upstream
     def test_invalid_properties(self):
         with self.assertRaises(exceptions.SchemaError):
-            self.Validator.check_schema({"properties": {"test": object()}})
+            self.Validator.check_schema({"properties": 12})
 
     def test_minItems_invalid_string(self):
         with self.assertRaises(exceptions.SchemaError):
@@ -1403,7 +1453,7 @@ class MetaSchemaTestsMixin(object):
         """
         Technically, all the spec says is they SHOULD have elements, not MUST.
 
-        See https://github.com/Julian/jsonschema/issues/529.
+        See #529.
         """
         self.Validator.check_schema({"enum": []})
 
@@ -1411,12 +1461,15 @@ class MetaSchemaTestsMixin(object):
         """
         Technically, all the spec says is they SHOULD be unique, not MUST.
 
-        See https://github.com/Julian/jsonschema/issues/529.
+        See #529.
         """
         self.Validator.check_schema({"enum": [12, 12]})
 
 
 class ValidatorTestMixin(MetaSchemaTestsMixin, object):
+    def test_it_implements_the_validator_protocol(self):
+        self.assertIsInstance(self.Validator({}), protocols.Validator)
+
     def test_valid_instances_are_valid(self):
         schema, instance = self.valid
         self.assertTrue(self.Validator(schema).is_valid(instance))
@@ -1442,13 +1495,48 @@ class ValidatorTestMixin(MetaSchemaTestsMixin, object):
         with self.assertRaises(exceptions.ValidationError):
             validator.validate(None)
 
+    def test_evolve(self):
+        ref, schema = "someCoolRef", {"type": "integer"}
+        resolver = validators.RefResolver("", {}, store={ref: schema})
+
+        validator = self.Validator(schema, resolver=resolver)
+        new = validator.evolve(schema={"type": "string"})
+
+        expected = self.Validator({"type": "string"}, resolver=resolver)
+
+        self.assertEqual(new, expected)
+        self.assertNotEqual(new, validator)
+
+    def test_evolve_with_subclass(self):
+        """
+        Subclassing validators isn't supported public API, but some users have
+        done it, because we don't actually error entirely when it's done :/
+
+        We need to deprecate doing so first to help as many of these users
+        ensure they can move to supported APIs, but this test ensures that in
+        the interim, we haven't broken those users.
+        """
+
+        with self.assertWarns(DeprecationWarning):
+            @attr.s
+            class OhNo(self.Validator):
+                foo = attr.ib(factory=lambda: [1, 2, 3])
+                _bar = attr.ib(default=37)
+
+        validator = OhNo({}, bar=12)
+        self.assertEqual(validator.foo, [1, 2, 3])
+
+        new = validator.evolve(schema={"type": "integer"})
+        self.assertEqual(new.foo, [1, 2, 3])
+        self.assertEqual(new._bar, 12)
+
     def test_it_delegates_to_a_legacy_ref_resolver(self):
         """
         Legacy RefResolvers support only the context manager form of
         resolution.
         """
 
-        class LegacyRefResolver(object):
+        class LegacyRefResolver:
             @contextmanager
             def resolving(this, ref):
                 self.assertEqual(ref, "the ref")
@@ -1541,7 +1629,7 @@ class ValidatorTestMixin(MetaSchemaTestsMixin, object):
         """
         A tuple instance properly formats validation errors for uniqueItems.
 
-        See https://github.com/Julian/jsonschema/pull/224
+        See #224
         """
         TupleValidator = validators.extend(
             self.Validator,
@@ -1621,7 +1709,7 @@ class ValidatorTestMixin(MetaSchemaTestsMixin, object):
                 validator.validate(instance)
 
 
-class AntiDraft6LeakMixin(object):
+class AntiDraft6LeakMixin:
     """
     Make sure functionality from draft 6 doesn't leak backwards in time.
     """
@@ -1653,7 +1741,7 @@ class AntiDraft6LeakMixin(object):
 
 class TestDraft3Validator(AntiDraft6LeakMixin, ValidatorTestMixin, TestCase):
     Validator = validators.Draft3Validator
-    valid = {}, {}
+    valid: tuple[dict, dict] = ({}, {})
     invalid = {"type": "integer"}, "foo"
 
     def test_any_type_is_valid_for_type_any(self):
@@ -1685,29 +1773,35 @@ class TestDraft3Validator(AntiDraft6LeakMixin, ValidatorTestMixin, TestCase):
 
 class TestDraft4Validator(AntiDraft6LeakMixin, ValidatorTestMixin, TestCase):
     Validator = validators.Draft4Validator
-    valid = {}, {}
+    valid: tuple[dict, dict] = ({}, {})
     invalid = {"type": "integer"}, "foo"
 
 
 class TestDraft6Validator(ValidatorTestMixin, TestCase):
     Validator = validators.Draft6Validator
-    valid = {}, {}
+    valid: tuple[dict, dict] = ({}, {})
     invalid = {"type": "integer"}, "foo"
 
 
 class TestDraft7Validator(ValidatorTestMixin, TestCase):
     Validator = validators.Draft7Validator
-    valid = {}, {}
+    valid: tuple[dict, dict] = ({}, {})
+    invalid = {"type": "integer"}, "foo"
+
+
+class TestDraft201909Validator(ValidatorTestMixin, TestCase):
+    Validator = validators.Draft201909Validator
+    valid: tuple[dict, dict] = ({}, {})
     invalid = {"type": "integer"}, "foo"
 
 
 class TestDraft202012Validator(ValidatorTestMixin, TestCase):
     Validator = validators.Draft202012Validator
-    valid = {}, {}
+    valid: tuple[dict, dict] = ({}, {})
     invalid = {"type": "integer"}, "foo"
 
 
-class TestValidatorFor(SynchronousTestCase):
+class TestValidatorFor(TestCase):
     def test_draft_3(self):
         schema = {"$schema": "http://json-schema.org/draft-03/schema"}
         self.assertIs(
@@ -1828,31 +1922,33 @@ class TestValidatorFor(SynchronousTestCase):
         self.assertIs(validators.validator_for({}, default=None), None)
 
     def test_warns_if_meta_schema_specified_was_not_found(self):
-        self.assertWarns(
-            category=DeprecationWarning,
-            message=(
-                "The metaschema specified by $schema was not found. "
-                "Using the latest draft to validate, but this will raise "
-                "an error in the future."
-            ),
-            # https://tm.tl/9363 :'(
-            filename=sys.modules[self.assertWarns.__module__].__file__,
+        with self.assertWarns(DeprecationWarning) as cm:
+            validators.validator_for(schema={"$schema": "unknownSchema"})
 
-            f=validators.validator_for,
-            schema={"$schema": "unknownSchema"},
-            default={},
+        self.assertEqual(cm.filename, __file__)
+        self.assertEqual(
+            str(cm.warning),
+            "The metaschema specified by $schema was not found. "
+            "Using the latest draft to validate, but this will raise "
+            "an error in the future.",
         )
 
     def test_does_not_warn_if_meta_schema_is_unspecified(self):
-        validators.validator_for(schema={}, default={})
-        self.assertFalse(self.flushWarnings())
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always")
+            validators.validator_for(schema={}, default={})
+        self.assertFalse(w)
+
+    def test_validator_for_custom_default_with_schema(self):
+        schema, default = {"$schema": "mailto:foo@example.com"}, object()
+        self.assertIs(validators.validator_for(schema, default), default)
 
 
-class TestValidate(SynchronousTestCase):
+class TestValidate(TestCase):
     def assertUses(self, schema, Validator):
         result = []
-        self.patch(Validator, "check_schema", result.append)
-        validators.validate({}, schema)
+        with mock.patch.object(Validator, "check_schema", result.append):
+            validators.validate({}, schema)
         self.assertEqual(result, [schema])
 
     def test_draft3_validator_is_chosen(self):
@@ -1934,14 +2030,18 @@ class TestValidate(SynchronousTestCase):
         )
 
     def test_it_uses_best_match(self):
-        # This is a schema that best_match will recurse into
-        schema = {"oneOf": [{"type": "string"}, {"type": "array"}]}
+        schema = {
+            "oneOf": [
+                {"type": "number", "minimum": 20},
+                {"type": "array"},
+            ],
+        }
         with self.assertRaises(exceptions.ValidationError) as e:
             validators.validate(12, schema)
-        self.assertIn("12 is not of type", str(e.exception))
+        self.assertIn("12 is less than the minimum of 20", str(e.exception))
 
 
-class TestRefResolver(SynchronousTestCase):
+class TestRefResolver(TestCase):
 
     base_uri = ""
     stored_uri = "foo://stored"
@@ -1956,14 +2056,11 @@ class TestRefResolver(SynchronousTestCase):
 
     def test_it_does_not_retrieve_schema_urls_from_the_network(self):
         ref = validators.Draft3Validator.META_SCHEMA["id"]
-        self.patch(
-            self.resolver,
-            "resolve_remote",
-            lambda *args, **kwargs: self.fail("Should not have been called!"),
-        )
-        with self.resolver.resolving(ref) as resolved:
-            pass
+        with mock.patch.object(self.resolver, "resolve_remote") as patched:
+            with self.resolver.resolving(ref) as resolved:
+                pass
         self.assertEqual(resolved, validators.Draft3Validator.META_SCHEMA)
+        self.assertFalse(patched.called)
 
     def test_it_resolves_local_refs(self):
         ref = "#/properties/foo"
@@ -2137,7 +2234,7 @@ def sorted_errors(errors):
 
 
 @attr.s
-class ReallyFakeRequests(object):
+class ReallyFakeRequests:
 
     _responses = attr.ib()
 
@@ -2149,7 +2246,7 @@ class ReallyFakeRequests(object):
 
 
 @attr.s
-class _ReallyFakeJSONResponse(object):
+class _ReallyFakeJSONResponse:
 
     _response = attr.ib()
 
